@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ContentHandler.cpp,v 1.77.2.6 2005/07/24 12:20:01 chpe Exp $
+ *  $Id: ContentHandler.cpp,v 1.83 2005/06/16 12:38:13 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -43,6 +43,7 @@
 #include <nsIMIMEInfo.h>
 #include <nsIInterfaceRequestorUtils.h>
 #include <nsCExternalHandlerService.h>
+#include <nsIDOMWindow.h>
 #include <nsNetError.h>
 
 #ifdef ALLOW_PRIVATE_API
@@ -298,7 +299,7 @@ NS_METHOD GContentHandler::MIMEConfirmAction ()
 			 /* translators: %s is the name of the application */
 			 _("Open this file with \"%s\"?"),
 			 mHelperApp->name);
-		gtk_message_dialog_format_secondary_text
+		gtk_message_dialog_format_secondary_markup
 			(GTK_MESSAGE_DIALOG (dialog),
 			 /* translators: %s is the name of the application */
 			 _("It's not possible to view this file type "
