@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.h,v 1.37.2.3 2005/08/13 12:58:33 chpe Exp $
+ *  $Id: EphyBrowser.h,v 1.44 2005/05/31 14:36:51 chpe Exp $
  */
 
 #ifndef EPHY_BROWSER_H
@@ -34,7 +34,6 @@
 #include <nsIWebNavigation.h>
 #include <nsISHistory.h>
 #include <nsIWebBrowser.h>
-#include <nsIWebBrowserFocus.h>
 #include <nsIDOMDocument.h>
 #include <nsIDOMWindow.h>
 #include <nsIPrintSettings.h>
@@ -53,37 +52,60 @@ class EphyBrowser;
 class EphyEventListener : public nsIDOMEventListener
 {
 public:
-	EphyEventListener();
-	virtual ~EphyEventListener();
-
-	nsresult Init (EphyBrowser *aOwner);
-
 	NS_DECL_ISUPPORTS
 
 	// nsIDOMEventListener
-
 	NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent) = 0;
 
+	EphyEventListener(EphyBrowser *aOwner) : mOwner(aOwner) { };
+	virtual ~EphyEventListener() { };
+
 protected:
 	EphyBrowser *mOwner;
 };
 
-class EphyFaviconEventListener : public EphyEventListener
+class EphyDOMLinkEventListener : public EphyEventListener
 {
 public:
 	NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent);
+
+	EphyDOMLinkEventListener(EphyBrowser *aOwner) : EphyEventListener(aOwner) { };
+private:
+	nsresult GetDocURI (nsIDOMElement *aElement,
+			    nsIURI **aDocURI);
+
 };
 
 class EphyPopupBlockEventListener : public EphyEventListener
 {
 public:
 	NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent);
+
+	EphyPopupBlockEventListener(EphyBrowser *aOwner) : EphyEventListener(aOwner) { };
 };
 
 class EphyModalAlertEventListener : public EphyEventListener
 {
 public:
 	NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent);
+
+	EphyModalAlertEventListener(EphyBrowser *aOwner) : EphyEventListener(aOwner) { };
+};
+
+class EphyDOMContentLoadedEventListener : public EphyEventListener
+{
+public:
+	NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent);
+
+	EphyDOMContentLoadedEventListener(EphyBrowser *aOwner) : EphyEventListener(aOwner) { };
+};
+
+class EphyDOMScrollEventListener : public EphyEventListener
+{
+public:
+	NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent);
+
+	EphyDOMScrollEventListener(EphyBrowser *aOwner) : EphyEventListener(aOwner) { };
 };
 
 class EphyContextMenuListener : public nsIDOMContextMenuListener
@@ -91,16 +113,13 @@ class EphyContextMenuListener : public nsIDOMContextMenuListener
 public:
         NS_DECL_ISUPPORTS
 
-        EphyContextMenuListener();   
-        virtual ~EphyContextMenuListener();
-
-        nsresult Init(EphyBrowser *aOwner);
-
 	// nsIDOMContextMenuListener
-
         NS_IMETHOD ContextMenu(nsIDOMEvent *aEvent);
         NS_IMETHOD HandleEvent(nsIDOMEvent *aEvent);
 
+	EphyContextMenuListener(EphyBrowser *aOwner) : mOwner(aOwner) { };
+	virtual ~EphyContextMenuListener() { };
+
 protected:
         EphyBrowser *mOwner;
 };
@@ -108,7 +127,9 @@ protected:
 class EphyBrowser
 {
 friend class EphyEventListener;
-friend class EphyFaviconEventListener;
+friend class EphyDOMLinkEventListener;
+friend class EphyDOMContentLoadedEventListener;
+friend class EphyDOMScrollEventListener;
 friend class EphyPopupBlockEventListener;
 friend class EphyModalAlertEventListener;
 friend class EphyContextMenuListener;
@@ -130,12 +151,6 @@ public:
 	nsresult PrintPreviewNumPages (int *numPages);
 	nsresult PrintPreviewNavigate(PRInt16 navType, PRInt32 pageNum);
 
-	nsresult FindSetProperties (const PRUnichar *search_string,
-			            PRBool case_sensitive,
-				    PRBool wrap_around);
-	nsresult Find (PRBool bacwards,
-		       PRBool *didFind);
-
 	nsresult GetPageDescriptor(nsISupports **aPageDescriptor);
 
 	nsresult GetSHInfo (PRInt32 *count, PRInt32 *index);
@@ -143,6 +158,14 @@ public:
 	nsresult GetSHUrlAtIndex (PRInt32 index, nsACString &url);
 	nsresult GoToHistoryIndex (PRInt16 index);
 
+#ifndef HAVE_GECKO_1_8
+	enum ReloadType { RELOAD_NORMAL		 = 0,
+			  RELOAD_FORCE		 = 1,
+			  RELOAD_ENCODING_CHANGE = 2 };
+
+	nsresult Reload (ReloadType flags);
+#endif
+
 	nsresult ForceEncoding (const char *encoding);
 	nsresult GetEncoding (nsACString &encoding);
 	nsresult GetForcedEncoding (nsACString &encoding);
@@ -163,20 +186,16 @@ public:
 
         EphyEmbedDocumentType GetDocumentType ();
 
-#ifdef GTKMOZEMBED_BROKEN_FOCUS
-	nsresult FocusActivate ();
-	nsresult FocusDeactivate ();
-#endif
-
 	nsCOMPtr<nsIWebBrowser> mWebBrowser;
 private:
 	GtkWidget *mEmbed;
 
-	nsCOMPtr<nsIWebBrowserFocus> mWebBrowserFocus;
 	nsCOMPtr<nsIDOMDocument> mTargetDocument;
 	nsCOMPtr<nsIDOMEventTarget> mEventTarget;
 	nsCOMPtr<nsIDOMWindow> mDOMWindow;
-	EphyFaviconEventListener *mFaviconEventListener;
+	EphyDOMLinkEventListener *mDOMLinkEventListener;
+	EphyDOMContentLoadedEventListener *mDOMContentLoadedEventListener;
+	EphyDOMScrollEventListener *mDOMScrollEventListener;
 	EphyPopupBlockEventListener *mPopupBlockEventListener;
 	EphyModalAlertEventListener *mModalAlertListener;
 	EphyContextMenuListener *mContextMenuListener;
