@@ -15,27 +15,36 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyUtils.h,v 1.5 2004/11/20 23:17:59 chpe Exp $
+ *  $Id: EphyUtils.h,v 1.8 2005/06/02 20:45:15 jframeau Exp $
  */
 
 #ifndef EPHY_UTILS_H
 #define EPHY_UTILS_H
 
-#include <nsIIOService.h>
-#include <nsIURI.h>
-#include <nsIDOMWindow.h>
+#include <nsError.h>
 #include <gtk/gtkwidget.h>
 
+class nsAString;
+class nsACString;
+class nsIDOMWindow;
+class nsIIOService;
 class nsIPrintSettings;
+class nsIURI;
 struct _EmbedPrintInfo;
 
 namespace EphyUtils
 {
 	nsresult	GetIOService		(nsIIOService **ioService);
 
-	nsresult	NewURI			(nsIURI **result, const nsAString &spec);
+	nsresult	NewURI			(nsIURI **result,
+						 const nsAString &spec,
+						 const char *charset = nsnull,
+						 nsIURI *baseURI = nsnull);
 
-	nsresult	NewURI			(nsIURI **result, const nsACString &spec);
+	nsresult	NewURI			(nsIURI **result,
+						 const nsACString &spec,
+						 const char *charset = nsnull,
+						 nsIURI *baseURI = nsnull);
 
 	GtkWidget      *FindEmbed		(nsIDOMWindow *aDOMWindow);
 
@@ -44,6 +53,8 @@ namespace EphyUtils
 	nsresult        CollatePrintSettings	(const _EmbedPrintInfo *info,
 						 nsIPrintSettings *settings,
 						 gboolean preview);
+	PRBool          StringEquals            (nsAString &s1, 
+						 nsAString &s2);
 }
 
 #endif
