@@ -16,14 +16,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: PrintingPromptService.cpp,v 1.18 2004/10/10 16:03:53 marco Exp $
+ *  $Id: PrintingPromptService.cpp,v 1.21 2005/07/05 11:33:54 chpe Exp $
  */
 
 #include "mozilla-config.h"
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <gtk/gtkdialog.h>
 
@@ -45,12 +43,12 @@ NS_IMPL_ISUPPORTS1(GPrintingPromptService, nsIPrintingPromptService)
 
 GPrintingPromptService::GPrintingPromptService()
 {
-	LOG ("GPrintingPromptService ctor (%p)", this)
+	LOG ("GPrintingPromptService ctor (%p)", this);
 }
 
 GPrintingPromptService::~GPrintingPromptService()
 {
-	LOG ("GPrintingPromptService dtor (%p)", this)
+	LOG ("GPrintingPromptService dtor (%p)", this);
 }
 
 /* void showPrintDialog (in nsIDOMWindow parent, in nsIWebBrowserPrint webBrowserPrint, in nsIPrintSettings printSettings); */
@@ -66,8 +64,7 @@ NS_IMETHODIMP GPrintingPromptService::ShowPrintDialog(nsIDOMWindow *parent, nsIW
 	EphyEmbed *embed = EPHY_EMBED (EphyUtils::FindEmbed (parent));
 	NS_ENSURE_TRUE (embed, NS_ERROR_FAILURE);
 
-	if (!(eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINT_SETUP) ||
-	      eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_COMMAND_LINE)))
+	if (!eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINT_SETUP))
 	{
 		GtkWidget *gtkParent = EphyUtils::FindGtkParent(parent);
 		NS_ENSURE_TRUE (gtkParent, NS_ERROR_FAILURE);
@@ -117,8 +114,7 @@ NS_IMETHODIMP GPrintingPromptService::ShowPageSetup(nsIDOMWindow *parent, nsIPri
 	nsresult rv = NS_ERROR_ABORT;
 
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINTING) ||
-	    eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINT_SETUP) ||
-	    eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_COMMAND_LINE))
+	    eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_PRINT_SETUP))
 	{
 		return rv;
 	}
