@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id$
+ *  $Id: mozilla-embed.cpp,v 1.109 2005/07/04 12:22:40 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -64,6 +64,9 @@ static gboolean mozilla_embed_dom_mouse_click_cb(GtkMozEmbed *embed,
 static gboolean mozilla_embed_dom_mouse_down_cb	(GtkMozEmbed *embed,
 						 gpointer dom_event, 
 						 MozillaEmbed *membed);
+static gboolean mozilla_embed_dom_key_press_cb	(GtkMozEmbed *embed,
+						 gpointer dom_event, 
+						 MozillaEmbed *membed);
 static void mozilla_embed_new_window_cb		(GtkMozEmbed *embed, 
 						 GtkMozEmbed **newEmbed,
 						 guint chrome_mask,
@@ -172,41 +175,12 @@ mozilla_embed_get_type (void)
         return type;
 }
 
-static gboolean
-impl_find_next (EphyEmbed *embed, 
-                gboolean backwards)
-{
-	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-	nsresult rv;
-        PRBool didFind;
-
-        rv = mpriv->browser->Find (backwards, &didFind);
-	
-	return NS_SUCCEEDED (rv) ? didFind : FALSE;
-}
-
 static void
 impl_activate (EphyEmbed *embed) 
 {
 	gtk_widget_grab_focus (GTK_BIN (embed)->child);
 }
 
-static void
-impl_find_set_properties (EphyEmbed *embed, 
-                          const char *search_string,
-	                  gboolean case_sensitive,
-			  gboolean wrap_around)
-{
-	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-
-	nsEmbedString searchString;
-	NS_CStringToUTF16 (nsEmbedCString(search_string),
-			   NS_CSTRING_ENCODING_UTF8, searchString);
-
-	mpriv->browser->FindSetProperties (searchString.get(), case_sensitive,
-					   wrap_around); 
-}
-
 static void
 mozilla_embed_realize (GtkWidget *widget)
 {
@@ -273,6 +247,9 @@ mozilla_embed_init (MozillaEmbed *embed)
 	g_signal_connect_object (G_OBJECT (embed), "dom_mouse_down",
 				 G_CALLBACK (mozilla_embed_dom_mouse_down_cb),
 				 embed, (GConnectFlags) 0);
+	g_signal_connect_object (G_OBJECT (embed), "dom-key-press",
+				 G_CALLBACK (mozilla_embed_dom_key_press_cb),
+				 embed, (GConnectFlags) 0);
 	g_signal_connect_object (G_OBJECT (embed), "new_window",
 				 G_CALLBACK (mozilla_embed_new_window_cb),
 				 embed, (GConnectFlags) 0);
@@ -513,7 +490,7 @@ static void
 impl_reload (EphyEmbed *embed, 
              gboolean force)
 {
-#ifndef GTKMOZEMBED_BROKEN_RELOAD
+#ifdef HAVE_GECKO_1_8
 	guint32 mflags;
 
 	mflags = GTK_MOZ_EMBED_FLAG_RELOADNORMAL;
@@ -728,7 +705,7 @@ impl_set_encoding (EphyEmbed *embed,
 		if (NS_FAILED (rv)) return;
 	}
 
-#ifndef GTKMOZEMBED_BROKEN_RELOAD
+#ifdef HAVE_GECKO_1_8
 	gtk_moz_embed_reload (GTK_MOZ_EMBED (embed),
 			      GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE);
 #else
@@ -943,6 +920,32 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 						"ge_dom_mouse_down");
 }
 
+static gint
+mozilla_embed_dom_key_press_cb (GtkMozEmbed *embed,
+				gpointer dom_event, 
+				MozillaEmbed *membed)
+{
+	gint retval = FALSE;
+
+	if (dom_event == NULL) return FALSE;
+
+	nsCOMPtr<nsIDOMKeyEvent> ev = static_cast<nsIDOMKeyEvent*>(dom_event);
+	NS_ENSURE_TRUE (ev, FALSE);
+
+	if (!EventContext::CheckKeyPress (ev)) return FALSE;
+
+	GdkEvent *event = gtk_get_current_event ();
+	if (event == NULL) return FALSE; /* shouldn't happen! */
+
+	g_return_val_if_fail (GDK_KEY_PRESS == event->type, FALSE);
+
+	g_signal_emit_by_name (embed, "ge-search-key-press", event, &retval);
+
+	gdk_event_free (event);
+
+	return retval;
+}
+
 EphyEmbedChrome
 _mozilla_embed_translate_chrome (GtkMozEmbedChromeFlags flags)
 {
@@ -1071,9 +1074,7 @@ ephy_embed_iface_init (EphyEmbedIface *iface)
 	iface->shistory_go_nth = impl_shistory_go_nth;
 	iface->get_security_level = impl_get_security_level;
 	iface->show_page_certificate = impl_show_page_certificate;
-	iface->find_next = impl_find_next;
 	iface->activate = impl_activate;
-	iface->find_set_properties = impl_find_set_properties;
 	iface->set_encoding = impl_set_encoding;
 	iface->get_encoding = impl_get_encoding;
 	iface->has_automatic_encoding = impl_has_automatic_encoding;
