@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-file-helpers.c,v 1.28 2005/04/19 11:14:40 chpe Exp $
+ *  $Id: ephy-file-helpers.c,v 1.29 2005/07/03 21:38:54 jframeau Exp $
  */
 
 #include "config.h"
@@ -416,19 +416,19 @@ load_mime_from_xml (void)
 		tag = xmlTextReaderConstName (reader);
 		type = xmlTextReaderNodeType (reader);
 
-		if (xmlStrEqual (tag, "safe") && type == XML_READER_TYPE_ELEMENT)
+		if (xmlStrEqual (tag, (const xmlChar *)"safe") && type == XML_READER_TYPE_ELEMENT)
 		{
 			permission = EPHY_MIME_PERMISSION_SAFE;
 		}
-		else if (xmlStrEqual (tag, "unsafe") && type == XML_READER_TYPE_ELEMENT)
+		else if (xmlStrEqual (tag, (const xmlChar *)"unsafe") && type == XML_READER_TYPE_ELEMENT)
 		{
 			permission = EPHY_MIME_PERMISSION_UNSAFE;
 		}
-		else if (xmlStrEqual (tag, "mime-type"))
+		else if (xmlStrEqual (tag, (const xmlChar *)"mime-type"))
 		{
 			xmlChar *type;
 
-			type = xmlTextReaderGetAttribute (reader, "type");
+			type = xmlTextReaderGetAttribute (reader, (const xmlChar *)"type");
 			g_hash_table_insert (mime_table, type,
 					     GINT_TO_POINTER (permission));
 		}
