@@ -1,5 +1,7 @@
 /*
  *  Copyright (C) 2002 Jorn Baayen
+ *  Copyright (C) 2003, 2004 Marco Pesenti Gritti
+ *  Copyright (C) 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,16 +17,24 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-file-helpers.h,v 1.11 2004/10/02 14:25:00 chpe Exp $
+ *  $Id: ephy-file-helpers.h,v 1.13 2005/02/26 22:01:48 chpe Exp $
  */
 
 #ifndef EPHY_FILE_HELPERS_H
 #define EPHY_FILE_HELPERS_H
 
 #include <glib.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
 G_BEGIN_DECLS
 
+typedef enum
+{
+	EPHY_MIME_PERMISSION_SAFE	= 1,
+	EPHY_MIME_PERMISSION_UNSAFE	= 2,
+	EPHY_MIME_PERMISSION_UNKNOWN	= 3
+} EphyMimePermission;
+
 const char *ephy_file                    (const char *filename);
 
 const char *ephy_dot_dir                 (void);
@@ -35,7 +45,7 @@ void        ephy_file_helpers_shutdown   (void);
 
 char       *ephy_file_downloads_dir      (void);
 
-char       *ephy_file_desktop_dir      (void);
+char       *ephy_file_desktop_dir	 (void);
 
 const char *ephy_file_tmp_dir	 	 (void);
 
@@ -53,6 +63,19 @@ gboolean    ephy_file_switch_temp_file   (const char *filename,
 
 void	    ephy_file_delete_on_exit	 (const char *path);
 
+EphyMimePermission ephy_file_check_mime	 (const char *mime_type);
+
+gboolean    ephy_file_launch_desktop_file (const char *filename,
+					   guint32 user_time);
+
+gboolean    ephy_file_launch_application (GnomeVFSMimeApplication *application,
+					  const char *parameter,
+					  guint32 user_time);
+
+gboolean    ephy_file_launch_handler	 (const char *mime_type,
+					  const char *address,
+					  guint32 user_time);
+
 G_END_DECLS
 
 #endif /* EPHY_FILE_HELPERS_H */
