@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-notebook.c,v 1.69 2005/02/01 14:47:33 chpe Exp $
+ *  $Id: ephy-notebook.c,v 1.75 2005/07/03 21:38:55 jframeau Exp $
  */
 
 #include "config.h"
@@ -59,6 +59,9 @@
 
 #define INSANE_NUMBER_OF_URLS 20
 
+/* Until https://bugzilla.mozilla.org/show_bug.cgi?id=296002 is fixed */
+#define KEEP_TAB_IN_SAME_TOPLEVEL
+
 #define EPHY_NOTEBOOK_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_NOTEBOOK, EphyNotebookPrivate))
 
 struct _EphyNotebookPrivate
@@ -67,9 +70,12 @@ struct _EphyNotebookPrivate
 	GtkTooltips *title_tips;
 	guint tabs_vis_notifier_id;
 	gulong motion_notify_handler_id;
-	gint x_start, y_start;
-	gboolean drag_in_progress;
-	gboolean show_tabs;
+	int x_start;
+	int y_start;
+
+	guint drag_in_progress : 1;
+	guint show_tabs : 1;
+	guint dnd_enabled : 1;
 };
 
 static void ephy_notebook_init           (EphyNotebook *notebook);
@@ -83,12 +89,18 @@ static void move_tab_to_another_notebook (EphyNotebook *src,
 /* Local variables */
 static GdkCursor *cursor = NULL;
 
-static GtkTargetEntry url_drag_types [] = 
+static const GtkTargetEntry url_drag_types [] = 
 {
         { EPHY_DND_URI_LIST_TYPE,   0, 0 },
         { EPHY_DND_URL_TYPE,        0, 1 }
 };
-static guint n_url_drag_types = G_N_ELEMENTS (url_drag_types);
+
+enum
+{
+	PROP_0,
+	PROP_DND_ENABLED,
+	PROP_SHOW_TABS
+};
 
 enum
 {
@@ -142,6 +154,57 @@ ephy_notebook_get_type (void)
 	return type;
 }
 
+void
+ephy_notebook_set_dnd_enabled (EphyNotebook *notebook,
+			       gboolean enabled)
+{
+	EphyNotebookPrivate *priv = notebook->priv;
+
+	priv->dnd_enabled = enabled;
+	/* FIXME abort any DNDs in progress */
+
+	g_object_notify (G_OBJECT (notebook), "dnd-enabled");
+}
+
+static void
+ephy_notebook_get_property (GObject *object,
+			    guint prop_id,
+			    GValue *value,
+			    GParamSpec *pspec)
+{
+	EphyNotebook *notebook = EPHY_NOTEBOOK (object);
+	EphyNotebookPrivate *priv = notebook->priv;
+
+	switch (prop_id)
+	{
+		case PROP_DND_ENABLED:
+			g_value_set_boolean (value, priv->dnd_enabled);
+			break;
+		case PROP_SHOW_TABS:
+			g_value_set_boolean (value, priv->show_tabs);
+			break;
+	}
+}
+
+static void
+ephy_notebook_set_property (GObject *object,
+			    guint prop_id,
+			    const GValue *value,
+			    GParamSpec *pspec)
+{
+	EphyNotebook *notebook = EPHY_NOTEBOOK (object);
+
+	switch (prop_id)
+	{
+		case PROP_DND_ENABLED:
+			ephy_notebook_set_dnd_enabled (notebook, g_value_get_boolean (value));
+			break;
+		case PROP_SHOW_TABS:
+			ephy_notebook_set_show_tabs (notebook, g_value_get_boolean (value));
+			break;
+	}
+}
+
 static void
 ephy_notebook_class_init (EphyNotebookClass *klass)
 {
@@ -150,6 +213,8 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = ephy_notebook_finalize;
+	object_class->get_property = ephy_notebook_get_property;
+	object_class->set_property = ephy_notebook_set_property;
 
 	signals[TAB_ADDED] =
 		g_signal_new ("tab_added",
@@ -201,7 +266,23 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 			      1,
 			      EPHY_TYPE_TAB);
 
-	g_type_class_add_private (object_class, sizeof(EphyNotebookPrivate));
+	g_object_class_install_property (object_class,
+					 PROP_DND_ENABLED,
+					 g_param_spec_boolean ("dnd-enabled",
+							       "DND enabled",
+							       "Whether the notebook allows tab reordering by DND",
+							       TRUE,
+							       G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class,
+					 PROP_SHOW_TABS,
+					 g_param_spec_boolean ("show-tabs",
+							       "Show tabs",
+							       "Whether the notebook shows the tabs bar",
+							       TRUE,
+							       G_PARAM_READWRITE));
+
+	g_type_class_add_private (object_class, sizeof (EphyNotebookPrivate));
 }
 
 static EphyNotebook *
@@ -343,6 +424,7 @@ ephy_notebook_move_tab (EphyNotebook *src,
 			g_signal_emit (G_OBJECT (src), signals[TABS_REORDERED], 0);
 		}
 	}
+#ifndef KEEP_TAB_IN_SAME_TOPLEVEL
 	else
 	{
 		/* make sure the tab isn't destroyed while we move it */
@@ -351,23 +433,28 @@ ephy_notebook_move_tab (EphyNotebook *src,
 		ephy_notebook_add_tab (dest, tab, dest_position, TRUE);
 		g_object_unref (tab);
 	}
+#endif
 }
 
 static void
-drag_start (EphyNotebook *notebook)
+drag_start (EphyNotebook *notebook,
+	    guint32 time)
 {
 	notebook->priv->drag_in_progress = TRUE;
 
 	/* get a new cursor, if necessary */
+	/* FIXME multi-head */
 	if (!cursor) cursor = gdk_cursor_new (GDK_FLEUR);
 
 	/* grab the pointer */
 	gtk_grab_add (GTK_WIDGET (notebook));
-	if (!gdk_pointer_is_grabbed ()) {
-		gdk_pointer_grab (GDK_WINDOW(GTK_WIDGET (notebook)->window),
+	/* FIXME multi-head */
+	if (!gdk_pointer_is_grabbed ())
+	{
+		gdk_pointer_grab (GTK_WIDGET (notebook)->window,
 				  FALSE,
 				  GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
-				  NULL, cursor, GDK_CURRENT_TIME);
+				  NULL, cursor, time);
 	}
 }
 
@@ -427,12 +514,11 @@ motion_notify_cb (EphyNotebook *notebook,
 					      notebook->priv->y_start,
 					      event->x_root, event->y_root))
 		{
-			drag_start (notebook);
-		}
-		else
-		{
-			return FALSE;
+			drag_start (notebook, event->time);
+			return TRUE;
 		}
+
+		return FALSE;
 	}
 
 	result = find_notebook_and_tab_at_pos ((gint)event->x_root,
@@ -462,12 +548,10 @@ move_tab_to_another_notebook (EphyNotebook *src,
 			      GdkEventMotion *event,
 			      int dest_position)
 {
+#ifndef KEEP_TAB_IN_SAME_TOPLEVEL
 	EphyTab *tab;
 	int cur_page;
 
-	LOG ("moving tab from notebook %p to notebook %p at %d",
-	     src, dest, dest_position);
-
 	/* This is getting tricky, the tab was dragged in a notebook
 	 * in another window of the same app, we move the tab
 	 * to that new notebook, and let this notebook handle the
@@ -476,6 +560,9 @@ move_tab_to_another_notebook (EphyNotebook *src,
 	g_assert (EPHY_IS_NOTEBOOK (dest));
 	g_assert (dest != src);
 
+	/* Check if the dest notebook can accept the drag */
+	if (!dest->priv->dnd_enabled) return;
+
 	cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (src));
 	tab = EPHY_TAB (gtk_notebook_get_nth_page (GTK_NOTEBOOK (src), cur_page));
 
@@ -491,13 +578,15 @@ move_tab_to_another_notebook (EphyNotebook *src,
 	ephy_notebook_move_tab (src, dest, tab, dest_position);
 
 	/* start drag handling in dest notebook */
-	drag_start (dest);
 
 	dest->priv->motion_notify_handler_id =
 		g_signal_connect (G_OBJECT (dest),
 				  "motion-notify-event",
 				  G_CALLBACK (motion_notify_cb),
 				  NULL);
+
+	drag_start (dest, event->time);
+#endif /* KEEP_TAB_IN_SAME_TOPLEVEL */
 }
 
 static gboolean
@@ -515,6 +604,7 @@ button_release_cb (EphyNotebook *notebook,
 		cur_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
 						      cur_page_num);
 
+#ifndef KEEP_TAB_IN_SAME_TOPLEVEL
 		if (!is_in_notebook_window (notebook, event->x_root, event->y_root)
 		    && gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook)) > 1)
 		{
@@ -522,13 +612,14 @@ button_release_cb (EphyNotebook *notebook,
 			g_signal_emit (G_OBJECT (notebook),
 				       signals[TAB_DETACHED], 0, cur_page);
 		}
+#endif
 
 		/* ungrab the pointer if it's grabbed */
 		if (gdk_pointer_is_grabbed ())
 		{
 			gdk_pointer_ungrab (event->time);
-			gtk_grab_remove (GTK_WIDGET (notebook));
 		}
+		gtk_grab_remove (GTK_WIDGET (notebook));
 	}
 
 	/* This must be called even if a drag isn't happening */
@@ -542,10 +633,13 @@ button_press_cb (EphyNotebook *notebook,
 		 GdkEventButton *event,
 		 gpointer data)
 {
-	gint tab_clicked = find_tab_num_at_pos (notebook,
-						event->x_root,
-						event->y_root);
+	int tab_clicked;
 
+	if (!notebook->priv->dnd_enabled) return FALSE;
+
+	tab_clicked = find_tab_num_at_pos (notebook, event->x_root, event->y_root);
+
+	/* FIXME: how could there be a drag in progress!? */
 	if (notebook->priv->drag_in_progress)
 	{
 		return TRUE;
@@ -631,7 +725,7 @@ notebook_drag_data_received_cb (GtkWidget* widget, GdkDragContext *context,
 		char **split;
 
 		/* URL_TYPE has format: url \n title */
-		split = g_strsplit (selection_data->data, "\n", 2);
+		split = g_strsplit ((const gchar *)selection_data->data, "\n", 1);
 		if (split != NULL && split[0] != NULL && split[0][0] != '\0')
 		{
 			ephy_link_open (EPHY_LINK (notebook), split[0], tab,
@@ -704,6 +798,7 @@ ephy_notebook_init (EphyNotebook *notebook)
 	gtk_object_sink (GTK_OBJECT (notebook->priv->title_tips));
 
 	notebook->priv->show_tabs = TRUE;
+	notebook->priv->dnd_enabled = TRUE;
 
 	g_signal_connect (notebook, "button-press-event",
 			  (GCallback)button_press_cb, NULL);
@@ -721,7 +816,7 @@ ephy_notebook_init (EphyNotebook *notebook)
 			  NULL);
         gtk_drag_dest_set (GTK_WIDGET(notebook), GTK_DEST_DEFAULT_MOTION |
 			   GTK_DEST_DEFAULT_DROP,
-                           url_drag_types,n_url_drag_types,
+                           url_drag_types, G_N_ELEMENTS (url_drag_types),
                            GDK_ACTION_MOVE | GDK_ACTION_COPY);
 
 	notebook->priv->tabs_vis_notifier_id = eel_gconf_notification_add
@@ -907,6 +1002,7 @@ build_tab_label (EphyNotebook *nb, EphyTab *tab)
 	/* setup label */
         label = gtk_label_new ("");
 	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+	gtk_label_set_single_line_mode (GTK_LABEL (label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
         gtk_misc_set_padding (GTK_MISC (label), 0, 0);
 	gtk_box_pack_start (GTK_BOX (label_hbox), label, TRUE, TRUE, 0);
@@ -961,7 +1057,7 @@ ephy_notebook_add_tab (EphyNotebook *nb,
 	g_signal_connect (G_OBJECT(label), "drag_data_received",
 			  G_CALLBACK(notebook_drag_data_received_cb), tab);
 	gtk_drag_dest_set (label, GTK_DEST_DEFAULT_ALL,
-			   url_drag_types,n_url_drag_types,
+			   url_drag_types, G_N_ELEMENTS (url_drag_types),
 			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
 
 	sync_icon (tab, NULL, label);
