@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-session.c,v 1.34 2005/01/08 16:40:45 chpe Exp $
+ *  $Id: ephy-session.c,v 1.42 2005/07/04 12:22:41 chpe Exp $
  */
 
 #include "config.h"
@@ -32,6 +32,7 @@
 #include "ephy-file-helpers.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
+#include "ephy-gui.h"
 #include "ephy-debug.h"
 
 #include <glib/gi18n.h>
@@ -56,7 +57,8 @@ struct _EphySessionPrivate
 	GList *windows;
 	GList *tool_windows;
 	GtkWidget *resume_dialog;
-	gboolean dont_save;
+	guint dont_save : 1;
+	guint quit_while_resuming : 1;
 };
 
 #define BOOKMARKS_EDITOR_ID	"BookmarksEditor"
@@ -190,7 +192,7 @@ window_focus_in_event_cb (EphyWindow *window,
 			  GdkEventFocus *event,
 			  EphySession *session)
 {
-	LOG ("focus-in-event for window %p", window)
+	LOG ("focus-in-event for window %p", window);
 
 	g_return_val_if_fail (g_list_find (session->priv->windows, window) != NULL, FALSE);
 
@@ -211,7 +213,7 @@ impl_attach_window (EphyExtension *extension,
 	EphySession *session = EPHY_SESSION (extension);
 	GtkWidget *notebook;
 
-	LOG ("impl_attach_window")
+	LOG ("impl_attach_window");
 
 	session->priv->windows = g_list_append (session->priv->windows, window);
 	ephy_session_save (session, SESSION_CRASHED);
@@ -226,6 +228,22 @@ impl_attach_window (EphyExtension *extension,
 			  G_CALLBACK (tab_removed_cb), session);
 	g_signal_connect (notebook, "tabs_reordered",
 			  G_CALLBACK (tabs_reordered_cb), session);
+
+	/* Set unique identifier as role, so that on restore, the WM can
+	 * place the window on the right workspace
+	 */
+
+	if (gtk_window_get_role (GTK_WINDOW (window)) == NULL)
+	{
+		/* I guess rand() is unique enough, otherwise we could use
+		 * time + pid or something
+		 */
+		char *role;
+
+		role = g_strdup_printf ("epiphany-window-%x", rand());
+		gtk_window_set_role (GTK_WINDOW (window), role);
+		g_free (role);
+	}
 }
 
 static void
@@ -234,7 +252,7 @@ impl_detach_window (EphyExtension *extension,
 {
 	EphySession *session = EPHY_SESSION (extension);
 
-	LOG ("impl_detach_window")
+	LOG ("impl_detach_window");
 
 	session->priv->windows = g_list_remove (session->priv->windows, window);
 	ephy_session_save (session, SESSION_CRASHED);
@@ -244,41 +262,29 @@ impl_detach_window (EphyExtension *extension,
 	 */
 }
 
-static void
-ensure_session_directory (void)
-{
-	char *dir;
-
-	dir = g_build_filename (ephy_dot_dir (), "sessions", NULL);
-	if (g_file_test (dir, G_FILE_TEST_EXISTS) == FALSE)
-	{
-		if (mkdir (dir, 488) != 0)
-		{
-			g_error ("Unable to create session directory '%s'\n", dir);
-		}
-	}
-
-	g_free (dir);
-}
-
 static void
 ephy_session_init (EphySession *session)
 {
 	session->priv = EPHY_SESSION_GET_PRIVATE (session);
 
-	LOG ("EphySession initialising")
-
-	ensure_session_directory ();
+	LOG ("EphySession initialising");
 }
 
 static void
 ephy_session_dispose (GObject *object)
 {
 	EphySession *session = EPHY_SESSION(object);
+	EphySessionPrivate *priv = session->priv;
 
-	LOG ("EphySession disposing")
+	LOG ("EphySession disposing");
 
-	session_delete (session, SESSION_CRASHED);
+	/* Only remove the crashed session if we're not shutting down while
+	 * the session resume dialogue was still shown!
+	*/
+	if (priv->quit_while_resuming == FALSE)
+	{
+		session_delete (session, SESSION_CRASHED);
+	}
 
 	parent_class->dispose (object);
 }
@@ -288,7 +294,7 @@ ephy_session_finalize (GObject *object)
 {
 	EphySession *session = EPHY_SESSION (object);
 
-	LOG ("EphySession finalising")
+	LOG ("EphySession finalising");
 
 	/* FIXME: those should be NULL already!? */
 	g_list_free (session->priv->windows);
@@ -357,7 +363,8 @@ ephy_session_class_init (EphySessionClass *class)
 }
 
 static gboolean
-offer_to_resume (EphySession *session)
+offer_to_resume (EphySession *session,
+		 guint32 user_time)
 {
 	GtkWidget *dialog;
 	int response;
@@ -385,7 +392,12 @@ offer_to_resume (EphySession *session)
 
 	session->priv->resume_dialog = dialog;
 
+	ephy_gui_window_update_user_time (session->priv->resume_dialog,
+					  user_time);
+
+	gtk_window_set_auto_startup_notification (FALSE);
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_window_set_auto_startup_notification (TRUE);
 
 	gtk_widget_destroy (dialog);
 
@@ -397,6 +409,7 @@ offer_to_resume (EphySession *session)
 /**
  * ephy_session_autoresume:
  * @session: a #EphySession
+ * @user_time: a timestamp, or 0
  *
  * Resume a crashed session when necessary (interactive)
  *
@@ -405,49 +418,72 @@ offer_to_resume (EphySession *session)
  * has been re-presented to the user.
  **/
 gboolean
-ephy_session_autoresume (EphySession *session)
+ephy_session_autoresume (EphySession *session,
+			 guint32 user_time)
 {
+	EphySessionPrivate *priv = session->priv;
 	char *saved_session;
 	gboolean retval = FALSE;
 
-	LOG ("ephy_session_autoresume")
+	LOG ("ephy_session_autoresume");
 
-	if (session->priv->windows != NULL || session->priv->tool_windows != NULL) return FALSE;
+	if (priv->windows != NULL || priv->tool_windows != NULL) return FALSE;
 
-	if (session->priv->resume_dialog)
+	if (priv->resume_dialog)
 	{
-		gtk_window_present (GTK_WINDOW (session->priv->resume_dialog));
+		ephy_gui_window_update_user_time (session->priv->resume_dialog,
+						  user_time);
+		ephy_gui_window_present (GTK_WINDOW (priv->resume_dialog), user_time);
 		return TRUE;
 	}
 
 	saved_session = get_session_filename (SESSION_CRASHED);
 
 	if (g_file_test (saved_session, G_FILE_TEST_EXISTS)
-	    && offer_to_resume (session))
+	    && offer_to_resume (session, user_time))
 	{
 		session->priv->dont_save = TRUE;
-		retval = ephy_session_load (session, saved_session);
+		retval = ephy_session_load (session, saved_session,
+					    0 /* since we've shown the dialogue */);
 		session->priv->dont_save = FALSE;
 		ephy_session_save (session, SESSION_CRASHED);
 	}
 
 	g_free (saved_session);
 
-	return retval;
+	/* ensure we don't open a blank window when quitting while resuming */
+	return retval || priv->quit_while_resuming;
+}
+
+static void
+close_dialog (GtkWidget *widget)
+{
+	if (GTK_IS_DIALOG (widget))
+	{
+		/* don't destroy them, someone might have a ref on them */
+		gtk_dialog_response (GTK_DIALOG (widget),
+				     GTK_RESPONSE_DELETE_EVENT);
+	}
 }
 
 void
 ephy_session_close (EphySession *session)
 {
+	EphySessionPrivate *priv = session->priv;
 	GList *windows;
 
-	LOG ("ephy_session_close")
+	LOG ("ephy_session_close");
 
 	/* we have to ref the shell or else we may get finalised between
 	 * destroying the windows and destroying the tool windows
 	 */
 	g_object_ref (ephy_shell);
-	session->priv->dont_save = TRUE;
+
+	priv->dont_save = TRUE;
+	/* need to set this up here while the dialogue hasn't been killed yet */
+	priv->quit_while_resuming = priv->resume_dialog != NULL;	
+
+	ephy_embed_shell_prepare_close (embed_shell);
 
 	windows	= ephy_session_get_windows (session);
 	g_list_foreach (windows, (GFunc) gtk_widget_destroy, NULL);
@@ -457,6 +493,17 @@ ephy_session_close (EphySession *session)
 	g_list_foreach (windows, (GFunc) gtk_widget_destroy, NULL);
 	g_list_free (windows);	
 
+	ephy_embed_shell_prepare_close (embed_shell);
+
+	/* there may still be windows open, like dialogues posed from
+	 * web pages, etc. Try to kill them, but be sure NOT to destroy
+	 * the gtkmozembed offscreen window!
+	 * Here, we just check if it's a dialogue and close it if it is one.
+	 */
+	windows = gtk_window_list_toplevels ();
+	g_list_foreach (windows, (GFunc) close_dialog, NULL);
+	g_list_free (windows);
+
 	session->priv->dont_save = FALSE;
 	g_object_unref (ephy_shell);
 }
@@ -545,6 +592,7 @@ write_ephy_window (xmlTextWriterPtr writer,
 		   EphyWindow *window)
 {
 	GList *tabs, *l;
+	const char *role;
 	int ret;
 
 	tabs = ephy_window_get_tabs (window);
@@ -560,6 +608,15 @@ write_ephy_window (xmlTextWriterPtr writer,
 	ret = write_window_geometry (writer, GTK_WINDOW (window));
 	if (ret < 0) return ret;
 
+	role = gtk_window_get_role (GTK_WINDOW (window));
+	if (role != NULL)
+	{
+		ret = xmlTextWriterWriteAttribute (writer, 
+						   (const xmlChar *)"role", 
+						   (const xmlChar *)role);
+		if (ret < 0) return ret;
+	}
+
 	for (l = tabs; l != NULL; l = l->next)
 	{
 		EphyTab *tab = EPHY_TAB(l->data);
@@ -587,7 +644,7 @@ ephy_session_save (EphySession *session,
 		return TRUE;
 	}
 
-	LOG ("ephy_sesion_save %s", filename)
+	LOG ("ephy_sesion_save %s", filename);
 
 	if (session->priv->windows == NULL && session->priv->tool_windows == NULL)
 	{
@@ -730,14 +787,23 @@ restore_geometry (GtkWindow *window,
 
 	if (success)
 	{
+		tmp = xmlGetProp (node, (xmlChar *)"role");
+		if (tmp != NULL)
+		{
+			gtk_window_set_role (GTK_WINDOW (window), (const char *)tmp);
+			xmlFree (tmp);
+		}
+
 		gtk_window_move (window, x, y);
 		gtk_window_set_default_size (window, width, height);
+		
 	}
 }
 /*
  * ephy_session_load:
  * @session: a #EphySession
  * @filename: the path of the source file
+ * @user_time: a timestamp, or 0
  *
  * Load a session from disk, restoring the windows and their state
  *
@@ -745,14 +811,15 @@ restore_geometry (GtkWindow *window,
  **/
 gboolean
 ephy_session_load (EphySession *session,
-		   const char *filename)
+		   const char *filename,
+		   guint32 user_time)
 {
 	xmlDocPtr doc;
         xmlNodePtr child;
 	GtkWidget *widget = NULL;
 	char *save_to;
 
-	LOG ("ephy_sesion_load %s", filename)
+	LOG ("ephy_sesion_load %s", filename);
 
 	save_to = get_session_filename (filename);
 
@@ -776,7 +843,10 @@ ephy_session_load (EphySession *session,
 			widget = GTK_WIDGET (ephy_window_new ());
 			restore_geometry (GTK_WINDOW (widget), child);
 			parse_embed (child->children, EPHY_WINDOW (widget));
-			gtk_window_present (GTK_WINDOW (widget));
+
+			ephy_gui_window_update_user_time (widget, user_time);
+
+			gtk_widget_show (widget);
 		}
 		else if (xmlStrEqual (child->name, (const xmlChar *) "toolwindow"))
 		{
@@ -800,7 +870,10 @@ ephy_session_load (EphySession *session,
 			}
 
 			restore_geometry (GTK_WINDOW (widget), child);
-			gtk_window_present (GTK_WINDOW (widget));
+
+			ephy_gui_window_update_user_time (widget, user_time);
+
+			gtk_widget_show (widget);
 		}
 
 		child = child->next;
@@ -837,7 +910,7 @@ void
 ephy_session_add_window (EphySession *session,
 			 GtkWindow *window)
 {
-	LOG ("ephy_session_add_window %p", window)
+	LOG ("ephy_session_add_window %p", window);
 
 	session->priv->tool_windows =
 		g_list_append (session->priv->tool_windows, window);
@@ -857,7 +930,7 @@ void
 ephy_session_remove_window (EphySession *session,
 			    GtkWindow *window)
 {
-	LOG ("ephy_session_remove_window %p", window)
+	LOG ("ephy_session_remove_window %p", window);
 
 	session->priv->tool_windows =
 		g_list_remove (session->priv->tool_windows, window);
@@ -873,20 +946,22 @@ ephy_session_remove_window (EphySession *session,
  * need to take an action (like opening an url) on
  * a window but you dont have a target window.
  *
- * Return value: the current active browser window, or NULL of there is none.
+ * Return value: the current active non-popup browser window, or NULL of there is none.
  **/
 EphyWindow *
 ephy_session_get_active_window (EphySession *session)
 {
-	GList *first;
+	EphyWindow *window = NULL;
+	GList *l;
 
 	g_return_val_if_fail (EPHY_IS_SESSION (session), NULL);
 
-	first = session->priv->windows;
-	if (first != NULL)
+	for (l = session->priv->windows; l != NULL; l = l->next)
 	{
-		return EPHY_WINDOW (first->data);
+		window = EPHY_WINDOW (l->data);
+
+		if (ephy_window_get_is_popup (window) == FALSE) break;
 	}
 
-	return NULL;
+	return window;
 }
