@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-statusbar.c,v 1.15 2005/01/08 16:40:45 chpe Exp $
+ *  $Id: ephy-statusbar.c,v 1.17 2005/01/30 20:09:58 chpe Exp $
  */
 
 #include "config.h"
@@ -107,7 +107,7 @@ create_statusbar_security_icon (EphyStatusbar *s)
 	gtk_container_add (GTK_CONTAINER (s->priv->security_evbox),
 			   GTK_WIDGET (s->priv->security_icon));
 
-	ephy_statusbar_set_security_state (s, FALSE, NULL);
+	ephy_statusbar_set_security_state (s, NULL, NULL);
 
 	gtk_widget_show_all (s->security_frame);
 
@@ -300,12 +300,12 @@ ephy_statusbar_set_progress (EphyStatusbar *statusbar,
 
 /**
  * ephy_statusbar_add_widget:
- * @statusbar: a #EphyStatusbar
+ * @statusbar: an #EphyStatusbar
  * @widget: a #GtkWidget
  * 
  * Adds the @widget to the statusbar. Use this function whenever you want to
  * add a widget to the statusbar. You can remove the widget again with
- * gtk_container_remove().
+ * ephy_statusbar_remove_widget().
  **/
 void
 ephy_statusbar_add_widget (EphyStatusbar *statusbar,
@@ -326,3 +326,22 @@ ephy_statusbar_add_widget (EphyStatusbar *statusbar,
 		gtk_frame_set_shadow_type (GTK_FRAME (widget), shadow);
 	}
 }
+
+/**
+ * ephy_statusbar_remove_widget:
+ * @statusbar: an #EphyStatusbar
+ * @widget: a #GtkWidget
+ *
+ * Removes @widget, which must have been added to @statusbar using
+ * ephy_statusbar_add_widget ().
+ */
+void
+ephy_statusbar_remove_widget (EphyStatusbar *statusbar,
+			      GtkWidget *widget)
+{
+	g_return_if_fail (EPHY_IS_STATUSBAR (statusbar));
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	gtk_container_remove (GTK_CONTAINER (statusbar->priv->icon_container),
+			      widget);
+}
