@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.139 2005/02/24 15:00:17 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.143 2005/07/10 19:15:42 chpe Exp $
  */
 
 #include "config.h"
@@ -45,7 +45,6 @@
 #include "ephy-link.h"
 
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkstock.h>
@@ -59,6 +58,10 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkuimanager.h>
 #include <gtk/gtkclipboard.h>
+#include <libgnomevfs/gnome-vfs-result.h>
+#include <libgnomevfs/gnome-vfs-monitor.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
 #include <string.h>
 
 #define EPHY_TAB_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TAB, EphyTabPrivate))
@@ -87,10 +90,16 @@ struct _EphyTabPrivate
 	GSList *shown_popups;
 	EphyTabNavigationFlags nav_flags;
 	EphyEmbedDocumentType document_type;
+	guint idle_resize_handler;
+
+	/* File watch */
+	GnomeVFSMonitorHandle *monitor;
+	guint reload_scheduled_id;	
 };
 
 static void ephy_tab_class_init		(EphyTabClass *klass);
 static void ephy_tab_init		(EphyTab *tab);
+static void ephy_tab_dispose		(GObject *object);
 static void ephy_tab_finalize		(GObject *object);
 
 enum
@@ -141,6 +150,7 @@ static guint	popup_blocker_n_hidden		(EphyTab *tab);
 static gboolean	ephy_tab_get_popups_allowed	(EphyTab *tab);
 static void	ephy_tab_set_popups_allowed	(EphyTab *tab,
 						 gboolean allowed);
+static void	ephy_tab_file_monitor_cancel	(EphyTab *tab);
 
 /* Class functions */
 
@@ -343,7 +353,8 @@ ephy_tab_class_init (EphyTabClass *class)
 
         parent_class = g_type_class_peek_parent (class);
 
-        object_class->finalize = ephy_tab_finalize;
+	object_class->dispose = ephy_tab_dispose;
+	object_class->finalize = ephy_tab_finalize;
 	object_class->get_property = ephy_tab_get_property;
 	object_class->set_property = ephy_tab_set_property;
 
@@ -632,16 +643,20 @@ popups_manager_show_all (EphyTab *tab)
 static char *
 popups_manager_new_window_info (EphyWindow *window)
 {
+	EphyTab *tab;
 	EphyEmbedChrome chrome;
+	gboolean is_popup;
 	char *features;
-	int w, h;
 
-	g_object_get (window, "chrome", &chrome, NULL);
-	gtk_window_get_size (GTK_WINDOW (window), &w, &h);
+	g_object_get (window, "chrome", &chrome, "is-popup", &is_popup, NULL);
+	g_return_val_if_fail (is_popup, g_strdup (""));
+
+	tab = ephy_window_get_active_tab (window);
+	g_return_val_if_fail (tab != NULL, g_strdup (""));
 
 	features = g_strdup_printf
 		("width=%d,height=%d,menubar=%d,status=%d,toolbar=%d",
-		 w, h,
+		 tab->priv->width, tab->priv->height,
 		 (chrome & EPHY_EMBED_CHROME_MENUBAR) > 0,
 		 (chrome & EPHY_EMBED_CHROME_STATUSBAR) > 0,
 		 (chrome & EPHY_EMBED_CHROME_TOOLBAR) > 0);
@@ -745,18 +760,34 @@ popups_manager_reset (EphyTab *tab)
 	g_object_notify (G_OBJECT (tab), "hidden-popup-count");
 }
 
+static void
+ephy_tab_dispose (GObject *object)
+{
+	EphyTab *tab = EPHY_TAB (object);
+	EphyTabPrivate *priv = tab->priv;
+
+	if (priv->idle_resize_handler != 0)
+	{
+		g_source_remove (priv->idle_resize_handler);
+		priv->idle_resize_handler = 0;
+	}
+
+	ephy_tab_file_monitor_cancel (tab);
+
+	G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 static void
 ephy_tab_finalize (GObject *object)
 {
-        EphyTab *tab = EPHY_TAB (object);
+	EphyTab *tab = EPHY_TAB (object);
+	EphyTabPrivate *priv = tab->priv;
 
-	g_idle_remove_by_data (tab);
-
-	g_free (tab->priv->title);
-	g_free (tab->priv->address);
-	g_free (tab->priv->icon_address);
-	g_free (tab->priv->link_message);
-	g_free (tab->priv->status_message);
+	g_free (priv->title);
+	g_free (priv->address);
+	g_free (priv->icon_address);
+	g_free (priv->link_message);
+	g_free (priv->status_message);
 	popups_manager_reset (tab);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -782,6 +813,17 @@ address_has_web_scheme (const char *address)
 	return has_web_scheme;
 }
 
+static gboolean
+let_me_resize_hack (EphyTab *tab)
+{
+	EphyTabPrivate *priv = tab->priv;
+
+	gtk_widget_set_size_request (GTK_WIDGET (tab), -1, -1);
+
+	priv->idle_resize_handler = 0;	
+	return FALSE;
+}
+
 /* Public functions */
 
 /**
@@ -937,6 +979,54 @@ ephy_tab_get_size (EphyTab *tab, int *width, int *height)
 	}
 }
 
+/**
+ * ephy_tab_set_size:
+ * @tab: an #EphyTab
+ * @width:
+ * @height:
+ *
+ * Sets the size of @tab. This is not guaranteed to actually resize the tab.
+ **/
+void
+ephy_tab_set_size (EphyTab *tab,
+		   int width,
+		   int height)
+{
+	EphyTabPrivate *priv = tab->priv;
+	GtkWidget *widget = GTK_WIDGET (tab);
+	GtkAllocation allocation;
+
+	priv->width = width;
+	priv->height = height;
+
+	gtk_widget_set_size_request (widget, width, height);
+
+	/* HACK: When the web site changes both width and height,
+	 * we will first get a width change, then a height change,
+	 * without actually resizing the window in between (since
+	 * that happens only on idle).
+	 * If we don't set the allocation, GtkMozEmbed doesn't tell
+	 * mozilla the new width, so the height change sets the width
+	 * back to the old value!
+	 */
+	allocation.x = widget->allocation.x;
+	allocation.y = widget->allocation.y;
+	allocation.width = width;
+	allocation.height = height;
+	gtk_widget_size_allocate (widget, &allocation);
+
+	/* HACK: reset widget requisition after the container
+	 * has been resized. It appears to be the only way
+	 * to have the window sized according to embed
+	 * size correctly.
+	 */
+	if (priv->idle_resize_handler == 0)
+	{
+		priv->idle_resize_handler =
+			g_idle_add ((GSourceFunc) let_me_resize_hack, tab);
+	}
+}
+
 /**
  * ephy_tab_get_visibility:
  * @tab: an #EphyTab
@@ -995,6 +1085,124 @@ ephy_tab_set_icon_address (EphyTab *tab, const char *address)
 	g_object_notify (G_OBJECT (tab), "icon");
 }
 
+static void
+ephy_tab_file_monitor_cancel (EphyTab *tab)
+{
+	EphyTabPrivate *priv = tab->priv;
+
+	if (priv->monitor != NULL)
+	{
+		LOG ("Cancelling file monitor");
+
+		gnome_vfs_monitor_cancel (priv->monitor);
+		priv->monitor = NULL;
+	}
+
+	if (priv->reload_scheduled_id != 0)
+	{
+		LOG ("Cancelling scheduled reload");
+
+		g_source_remove (priv->reload_scheduled_id);
+		priv->reload_scheduled_id = 0;
+	}
+}
+
+static gboolean
+ephy_file_monitor_reload_cb (EphyTab *tab)
+{
+	EphyTabPrivate *priv = tab->priv;
+	EphyEmbed *embed;
+
+	LOG ("Reloading file '%s'\n", priv->address);
+
+	priv->reload_scheduled_id = 0;
+
+	embed = ephy_tab_get_embed (tab);
+	if (embed == NULL) return FALSE;
+
+	ephy_embed_reload (embed, TRUE);
+
+	/* don't run again */
+	return FALSE;
+}
+
+static void
+ephy_tab_file_monitor_cb (GnomeVFSMonitorHandle *handle,
+			  const gchar *monitor_uri,
+			  const gchar *info_uri,
+			  GnomeVFSMonitorEventType event_type,
+			  EphyTab *tab)
+{
+	EphyTabPrivate *priv = tab->priv;
+
+	LOG ("File '%s' has changed, scheduling reload", monitor_uri);
+
+	switch (event_type)
+	{
+		case GNOME_VFS_MONITOR_EVENT_CHANGED:
+		case GNOME_VFS_MONITOR_EVENT_CREATED:
+			/* We make a lot of assumptions here, but basically we know
+			 * that we just have to reload, by construction.
+			 * Delay the reload a little bit so we don't endlessly
+			 * reload while a file is written.
+			 */
+			if (priv->reload_scheduled_id != 0)
+			{
+				g_source_remove (priv->reload_scheduled_id);
+			}
+
+			priv->reload_scheduled_id =
+				g_timeout_add (100 /* ms */,
+					       (GSourceFunc) ephy_file_monitor_reload_cb,
+					       tab);
+			break;
+
+		case GNOME_VFS_MONITOR_EVENT_DELETED:
+		case GNOME_VFS_MONITOR_EVENT_STARTEXECUTING:
+		case GNOME_VFS_MONITOR_EVENT_STOPEXECUTING:
+		case GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED:
+		default:
+			break;
+	}
+}
+
+static void
+ephy_tab_update_file_monitor (EphyTab *tab,
+			      const gchar *address)
+{
+	EphyTabPrivate *priv = tab->priv;
+	GnomeVFSMonitorHandle *handle = NULL;
+	GnomeVFSURI *uri;
+	gboolean local;
+
+	if (priv->address != NULL && address != NULL &&
+	    strcmp (priv->address, address) == 0)
+	{
+		/* same address, no change needed */
+		return;
+	}
+
+	ephy_tab_file_monitor_cancel (tab);
+
+	uri = gnome_vfs_uri_new (address);
+	if (uri == NULL) return;
+
+	local = gnome_vfs_uri_is_local (uri);
+	gnome_vfs_uri_unref (uri);
+
+	if (local == FALSE) return;
+	
+	if (gnome_vfs_monitor_add (&handle, address,
+				   GNOME_VFS_MONITOR_FILE,
+				   (GnomeVFSMonitorCallback) ephy_tab_file_monitor_cb,
+				   tab) == GNOME_VFS_OK)
+	{
+		LOG ("Installed monitor for file '%s'", address);
+
+		priv->monitor = handle;
+	}
+}
+
 /**
  * ephy_tab_get_icon_address:
  * @tab: an #EphyTab
@@ -1029,12 +1237,18 @@ ephy_tab_link_message_cb (EphyEmbed *embed,
 }
 
 static void
-ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
+ephy_tab_address_cb (EphyEmbed *embed,
+		     const char *address,
+		     EphyTab *tab)
 {
 	const char *uv_address;
+	char *freeme = NULL;
 
 	LOG ("ephy_tab_address_cb tab %p address %s", tab, address);
 
+	/* do this up here so we still have the old address around */
+	ephy_tab_update_file_monitor (tab, address);
+
 	/* Do not expose about:blank to the user, an empty address
 	   bar will do better */
 	if (address && strcmp (address, "about:blank") == 0)
@@ -1043,7 +1257,7 @@ ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 	}
 	else
 	{
-		uv_address = address;
+		uv_address = freeme = ephy_embed_get_location (embed, TRUE);
 	}
 
 	if (tab->priv->address_expire == EPHY_TAB_ADDRESS_EXPIRE_NOW)
@@ -1054,6 +1268,8 @@ ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 	ephy_tab_set_link_message (tab, NULL);
 	ephy_tab_set_icon_address (tab, NULL);
 	ephy_tab_update_navigation_flags (tab, embed);
+
+	g_free (freeme);
 }
 
 static void
@@ -1330,14 +1546,6 @@ ephy_tab_popup_blocked_cb (EphyEmbed *embed, const char *url,
 	popups_manager_add (tab, url, features);
 }
 
-static gboolean
-let_me_resize_hack (GtkWidget *tab)
-{
-	gtk_widget_set_size_request (tab, -1, -1);
-
-	return FALSE;
-}
-
 static void
 ephy_tab_visibility_cb (EphyEmbed *embed, gboolean visibility,
 			EphyTab *tab)
@@ -1379,42 +1587,6 @@ ephy_tab_destroy_brsr_cb (EphyEmbed *embed, EphyTab *tab)
 	ephy_notebook_remove_tab (EPHY_NOTEBOOK (notebook), tab);
 }
 
-static void
-ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
-		     EphyTab *tab)
-{
-	GtkWidget *notebook;
-	EphyWindow *window;
-
-	LOG ("ephy_tab_size_to_cb tab %p width %d height %d", tab, width, height);
-
-	tab->priv->width = width;
-	tab->priv->height = height;
-
-	window = ephy_tab_get_window (tab);
-	notebook = ephy_window_get_notebook (window);
-
-	/* Do not resize window with multiple tabs.
-	 * Do not resize window already showed because
-	 * it's not possible to calculate a sensible window
-	 * size based on the embed size */
-	if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook)) == 1 &&
-	    !tab->priv->visibility)
-	{
-		gtk_widget_set_size_request
-			(GTK_WIDGET (tab), width, height);
-
-		/* HACK reset widget requisition after the container
-		 * has been resized. It appears to be the only way
-		 * to have the window sized according to embed
-		 * size correctly.
-		 */
-
-		g_idle_add ((GSourceFunc) let_me_resize_hack, tab);
-	}
-}
-
-
 static gboolean
 open_link_in_new_tab (EphyTab *tab,
 		      const char *link_address)
@@ -1637,9 +1809,6 @@ ephy_tab_init (EphyTab *tab)
 	g_signal_connect_object (embed, "destroy_browser",
 				 G_CALLBACK (ephy_tab_destroy_brsr_cb),
 				 tab, 0);
-	g_signal_connect_object (embed, "size_to",
-				 G_CALLBACK (ephy_tab_size_to_cb),
-				 tab, 0);
 	g_signal_connect_object (embed, "ge_dom_mouse_click",
 				 G_CALLBACK (ephy_tab_dom_mouse_click_cb),
 				 tab, 0);
@@ -1883,7 +2052,7 @@ ephy_tab_set_location (EphyTab *tab,
 {
 	g_return_if_fail (EPHY_IS_TAB (tab));
 
-	if (tab->priv->address) g_free (tab->priv->address);
+	g_free (tab->priv->address);
 	tab->priv->address = g_strdup (address);
 
 	if (expire == EPHY_TAB_ADDRESS_EXPIRE_CURRENT &&
